/*!\file openCV.cpp
 * \brief segmentation d'images.
 *
 * \author Farès BELHADJ, amsi@ai.univ-paris8.fr
 * \date September 13 2018
 */

#include <opencv2/opencv.hpp>
#include <iostream>
#include <segmentation.h>

using namespace cv;
using namespace std;

static double _getElapsedTime(void);

static int _hsvSeg = 0;

static void _bgr2hsv(Mat& img) {
  int i, j, ii, jj;
  double r, g, b, h, s, v;
  uint8_t * dd = img.data;
  assert(img.channels() == 3);
  for(i = 0; i < img.rows; ++i) {
    ii = i * img.step;
    for(j = 0; j < img.cols; ++j) {
      jj = ii + j * img.channels();
      b = dd[jj + 0] / 255.0;
      g = dd[jj + 1] / 255.0;
      r = dd[jj + 2] / 255.0;
      rgb2hsv(r, g, b, &h, &s, &v);
      dd[jj + 0] = h * 255.0;
      dd[jj + 1] = s * 255.0;
      dd[jj + 2] = v * 255.0;
    }
  }
}

static void _hsv2bgr(Mat& img) {
  int i, j, ii, jj;
  double r, g, b, h, s, v;
  uint8_t * dd = img.data;
  assert(img.channels() == 3);
  for(i = 0; i < img.rows; ++i) {
    ii = i * img.step;
    for(j = 0; j < img.cols; ++j) {
      jj = ii + j * img.channels();
      h = dd[jj + 0] / 255.0;
      s = dd[jj + 1] / 255.0;
      v = dd[jj + 2] / 255.0;
      hsv2rgb(h, s, v, &r, &g, &b);
      dd[jj + 0] = b * 255.0;
      dd[jj + 1] = g * 255.0;
      dd[jj + 2] = r * 255.0;
    }
  }
}

static void _segmenter_basic(Mat& img) {
  assert(img.channels() == 3);
  segmentUsingKmean(img.data, img.cols, img.rows, img.step, img.channels(), 5, 2); //R
  segmentUsingKmean(img.data, img.cols, img.rows, img.step, img.channels(), 5, 1); //G
  segmentUsingKmean(img.data, img.cols, img.rows, img.step, img.channels(), 5, 0); //B
}

static void _segmenter_hsv(Mat& img) {
  assert(img.channels() == 3);
  _bgr2hsv(img);
  segmentUsingKmean(img.data, img.cols, img.rows, img.step, img.channels(), 5, 2); //V
  _hsv2bgr(img);
}

int main(int argc, char ** argv) {
  uint32_t frames = 0;
  double t0, t, dt;
  /* essayer d'ouvrir la seconde caméra */
  VideoCapture cap(1);
  /* si echec */
  if(!cap.isOpened()) {
    /* essayer d'ouvrir n'importe quelle caméra */
    cap.open(CV_CAP_ANY);
    /* si echec, quitte */
    if(!cap.isOpened())
      return 1;
  }
  /* on récupère et imprime les dimensions de la vidéo */
  Size s(cap.get(CV_CAP_PROP_FRAME_WIDTH), cap.get(CV_CAP_PROP_FRAME_HEIGHT));
  cerr << "dimensions du flux " << s.width << "x" << s.height << endl;
  /* créer une fenêtre */
  namedWindow("Segmentation", WINDOW_AUTOSIZE);
  /* une image */
  Mat frame;
  /* boucle principale */
  for(t0 = _getElapsedTime();;) {
    /* récupérer la frame courante */
    cap >> frame;
    GaussianBlur(frame, frame, Size(5,5), 0, 0);
    if(_hsvSeg)
      _segmenter_hsv(frame);
    else
      _segmenter_basic(frame);
    /* afficher l'image */
    imshow("Segmentation", frame);
    /* attendre 10ms pour capture clavier, si echap alors on quitte,
       si 'v' on change de mode de segmentation */
    switch(cvWaitKey(10) & 0xFF) {
    case 27:
      return 0;
    case 'v':
      _hsvSeg = !_hsvSeg;
      break;
    }
    t = _getElapsedTime();
    frames++;
    if((dt = t - t0) > 5000) {
      cerr << "FPS = " << 1000.0 * frames / dt << endl;
      t0 = t;
      frames = 0;
    }
  }
  return 0;
}

/* POUR MESURER LE TEMPS */
#include <sys/time.h>

static struct timeval _t0;

static double _getElapsedTime_sub1(void);
static double _getElapsedTime_sub2(void);

static double (* _getET)(void) = _getElapsedTime_sub1;

static double _getElapsedTime_sub1(void) {
  gettimeofday(&_t0, (struct timezone*) 0);
  _getET = _getElapsedTime_sub2;
  return 0.0;
}

static double _getElapsedTime_sub2(void) {
  struct timeval t;
  double diff;
  gettimeofday(&t, (struct timezone*) 0);
  diff = (t.tv_sec - _t0.tv_sec) * 1000000
    + (t.tv_usec - _t0.tv_usec);
  return diff/1000.;
}
static double _getElapsedTime(void) {
  return _getET();
}

